
#include "IceClient.hpp"
#include "Logging.hpp"
#include <matt.hpp>
#include <Ice/Ice.h>

#ifdef WIN32
#define DELAY(secs) Sleep(secs*1000);
#else 
#define DELAY(secs) sleep(secs);
#endif

namespace {
  namespace detail {

    class IceParticipant : public ::matt::roles::Participant {

      matt::messaging::MailboxPrx mailbox;

      void setMailbox(
        matt::messaging::MailboxPrx const &,
        Ice::Current const &);

    public:

      matt::messaging::MailboxPrx getMailbox();
    };
  }
}

void detail::IceParticipant::setMailbox(
  matt::messaging::MailboxPrx const & m,
  Ice::Current const &) {
  mailbox = m;
}

matt::messaging::MailboxPrx 
detail::IceParticipant::getMailbox() {
  return mailbox;
}

matt::messaging::MailboxPrx 
matt::IceClient::getMailbox() {
  return static_cast< detail::IceParticipant & >
    (*participant).getMailbox();
}

matt::IceClient::IceClient(
  Ice::CommunicatorPtr const & c) 
: adapter(c->createObjectAdapter("matt.client")),
  participant(new detail::IceParticipant()) {

  matt::roles::ParticipantPrx const proxy = 
    matt::roles::ParticipantPrx::checkedCast(
      adapter->addWithUUID(participant));

  adapter->activate();

  matt::roles::MediatorPrx const mediator = 
    matt::roles::MediatorPrx::checkedCast(
      c->propertyToProxy("matt.server"));

  mediator->registerParticipant(proxy);
}

void matt::IceClient::run() {
  matt::messaging::MailboxPrx const mailbox = getMailbox();
  DELAY(1);
  if (mailbox->active()) {
    for(int i = 0; i != 10; ++i) { 
      if (mailbox->empty()) {
        std::string const & msg = "Hello?";
        MATT_DEBUG_LOG("Sending message \"" + msg + "\""); 
        mailbox->write(msg);
        MATT_DEBUG_LOG("Message sent");
      }
      else {
        MATT_DEBUG_LOG("Reading message");
        std::string const & msg = mailbox->read();
        MATT_DEBUG_LOG("Received \"" + msg + "\"");
      }
      DELAY(1);
    }
    MATT_DEBUG_LOG("Deactivating mailbox");
    mailbox->deactivate();
  }
}

