
#include "Logging.hpp"
#include "Application.hpp"
#include "IceServer.hpp"
#include "IceClient.hpp"

#ifdef WIN32 

# define WAIT_FOR_USER_TO_KILL_PROMPT \
    system("pause");

# define APPLICATION_EXTENSION "exe"
# define APPLICATION_NAME(x) x "." APPLICATION_EXTENSION
# define DIR_DELIMITER '\\'

#else

# define WAIT_FOR_USER_TO_KILL_PROMPT
# define APPLICATION_EXTENSION "tsk"
# define APPLICATION_NAME(x) x "." APPLICATION_EXTENSION
# define DIR_DELIMITER '/'

#endif

namespace {

  enum AppType {
    AppTypeServer,
    AppTypeClient
  };

  AppType getAppType(
    std::string const & str) {
    static ::std::string const server = APPLICATION_NAME("server");
    static ::std::string const client = APPLICATION_NAME("client");
    ::std::string::size_type begin = str.rfind(DIR_DELIMITER);

    if (begin == ::std::string::npos) {
      begin = 0;
    }
    ::std::string const app = str.substr(begin+1);
    AppType result = AppTypeClient;

    if (server == app) {
      result = AppTypeServer;
    }
    return result;
  }
}

int main(
  int argc, 
  char * argv[]) {

  int result = 255;

  matt::BaseApplication * app = 0;

  switch (getAppType(argv[0])) {
  case AppTypeServer: 
    MATT_DEBUG_LOG("Starting an IceServer");
    app = new ::matt::Application< matt::IceServer >();
    break;
  case AppTypeClient:
    MATT_DEBUG_LOG("Starting an IceClient");
    app = new ::matt::Application< matt::IceClient >();
    break;
  };
  if (app) {
    MATT_DEBUG_LOG("Starting Application main routine");
    result = app->main(argc, argv);
  }
  MATT_DEBUG_LOG("Stopping Application");
  WAIT_FOR_USER_TO_KILL_PROMPT;
  return result;
}

