package matt.protocol;

import javax.management.RuntimeErrorException;

import matt.MattAlreadyListeningException;
import matt.MattListenException;
import matt.MattNoMessageException;
import matt.MattNotActiveException;
import matt.MattNotListeningException;
import matt.messaging.EventListenerPrx;
import matt.messaging.MailboxPrx;

public class Mailbox {

  class InternalNotification {
  }

  InternalNotification mailboxAssigned = new InternalNotification();
  InternalNotification mailboxDeactive = new InternalNotification(); 
  
  MailboxPrx mailbox = null;

  void setMailbox(
    MailboxPrx mailbox) {
    synchronized (mailboxAssigned) {
      if (this.mailbox == null) {
        this.mailbox = mailbox;
        mailboxAssigned.notifyAll();
      }
      else {
        // TODO error throw something
      }
    }
  }

  private void waitForMailboxAssignment()
    throws InterruptedException {

    synchronized (mailboxAssigned) {
      if (mailbox == null) {
        mailboxAssigned.wait();
      }
      if (mailbox == null) {
        throw new RuntimeErrorException(new Error(
          "Mailbox notified but with no vaild assignment."));
      }
    }
  }

  public boolean active()
    throws InterruptedException {
    waitForMailboxAssignment();
    return mailbox.active();
  }

  public void subscribe(
    EventListenerPrx listener)
    throws MattAlreadyListeningException,
      MattListenException,
      MattNotActiveException,
      InterruptedException {

    waitForMailboxAssignment();
    mailbox.addListener(listener);
  }

  public void deactivate()
    throws InterruptedException {

    waitForMailboxAssignment();
    mailbox.deactivate();
  }

  public boolean empty()
    throws InterruptedException {

    waitForMailboxAssignment();
    return mailbox.empty();
  }

  public String receiveMail()
    throws MattNoMessageException,
      MattNotActiveException,
      InterruptedException {

    waitForMailboxAssignment();
    return mailbox.read();
  }

  public void unsubscribe()
    throws MattListenException,
      MattNotActiveException,
      MattNotListeningException,
      InterruptedException {

    waitForMailboxAssignment();
    mailbox.removeListener();
  }

  public void sendMail(
    String message)
    throws MattNotActiveException,
      InterruptedException {
    
    waitForMailboxAssignment();
    try {
      mailbox.write(message);
    }
    catch(final MattNotActiveException e) {
      // TODO
      e.printStackTrace();
      
      synchronized(this) {
        mailboxDeactive.notifyAll();
      }
    }
  }

  public void whileActive() throws InterruptedException {
    synchronized(mailboxDeactive) {
      mailboxDeactive.wait();
    }
  }
  
  public void dispose() throws InterruptedException {
    synchronized(mailboxDeactive) {
      mailboxDeactive.notifyAll();
    }
    deactivate();
  }
}
