package matt.protocol;

import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import matt.roles.MediatorPrx;
import matt.roles.MediatorPrxHelper;
import matt.roles.ParticipantPrx;
import matt.roles.ParticipantPrxHelper;
import Ice.Communicator;
import Ice.ObjectAdapter;

public class Runtime {

  static final String PARAMETER_REGEX = "--Matt\\.[\\w\\.]+=[\\W]*[^\\W]";
  static final Pattern PARAMETER_PARSER = Pattern.compile(PARAMETER_REGEX);

  private final Communicator communicator;
  private final Properties properties;

  public Runtime(
    final String[] args) {
    communicator = Ice.Util.initialize(args);
    properties = parseArgs(args);
  }

  static private final Properties parseArgs(
    final String[] args) {
    final Properties properties = new Properties();
    for (final String arg : args) {
      final Matcher raw_pair = PARAMETER_PARSER.matcher(arg);
      if (raw_pair.find()) {
        final int name_start = raw_pair.start() + 2; // skip the "--"
        final int equal_start = arg.indexOf('=');
        final int value_start = raw_pair.end();
        final String name = arg.substring(name_start, equal_start);
        final String value = arg.substring(equal_start + 1, value_start);
        properties.setProperty(name, value);
      }
    }
    return properties;
  }

  private final ParticipantPrx createParticipantPrx(
    final Participant client) {

    ObjectAdapter adapter = communicator.createObjectAdapter("matt.client");
    adapter.activate();

    return ParticipantPrxHelper.checkedCast(adapter
      .addWithUUID(client.implemtnation));
  }

  private final MediatorPrx createMediatorPrx() {
    return MediatorPrxHelper.checkedCast(communicator
      .propertyToProxy("matt.server"));
  }

  public void register(
    final Participant client) {
    ParticipantPrx participant = createParticipantPrx(client);
    MediatorPrx mediator = createMediatorPrx();
    mediator.registerParticipant(participant);
  }

  public String getProperty(
    final String name) {
    return properties.getProperty(name);
  }

  public String getProperty(
    final String name,
    final String defaultValue) {
    return properties.getProperty(name, defaultValue);
  }

  public void dispose() {
    communicator.destroy();
  }
}
